package runtime;

import engine.Constants;
import guiAgent.GUI;
import guiAgent.GUIPlayer;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class PlayerManual implements IPlayer, GUIPlayer {

	private final Lock lock = new ReentrantLock();
	private final Condition clickedEvent = lock.newCondition();
	private int move;

	private GUI gui;
	private boolean myTurn = false;
	private boolean inGame = false;
	
	public PlayerManual() {
		gui = new GUI(PlayerManual.this);
	}
	
	/**
	 * Function that the gui is expected to call once it's made a move (i.e. the user clicked), or the knowledge agent once it's decided what move to make
	 */
	public void setMove(int move) {
		this.move = move;
		
		lock.lock();
		clickedEvent.signal();
		lock.unlock();
	}
	
	/**
	 * Returns a boolean value indicating if it's the agent's turn to move
	 */
	public boolean isMyTurn() {
		return myTurn;
	}

	/**
	 * Returns a boolean value indicating if the agent is in the middle of the game
	 */
	public boolean isInGame() {
		return inGame;
	}

	@Override
	public int getMove() {

		myTurn = true;
		move = -1;
		
		lock.lock();
		try {
			while (gui.shell==null) // hack instead of proper thread control. this waits for initialisation to finish
				Thread.sleep(300);

			while(!gui.shell.isDisposed() && Manager.isActive() && move==-1)
				clickedEvent.await(200, TimeUnit.MILLISECONDS);
			
		} catch (InterruptedException e) {
			//e.printStackTrace();
		}
		lock.unlock();

		if (!Manager.isActive()) {
			dispose();
			return 0;
		}
		if (gui.shell.isDisposed())
			return 0;

		myTurn = false;
		gui.move(move, false);

		return move-1;
	}

	public void dispose() {
		gui.dispose();
	}

	@Override
	public void setOpponentsMove(int move) {
		gui.move(move+1, true);
	}
	
	/**
	 * 
	 * @return a boolean value indicating if manual player's gui is active
	 */
	public boolean isActive() {
		try {
			while (gui.shell==null) // hack instead of proper thread control. this waits for initialisation to finish
				Thread.sleep(300);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return !gui.shell.isDisposed(); 
	}

	@Override
	public void start(boolean doIStart) {
		myTurn = doIStart;
		inGame = true;
		gui.start(doIStart);
	}

	@Override
	public void finish(int didIWin) {
		inGame = false;
		gui.finish(didIWin);
	}

	@Override
	public int getNRows() {
		return Constants.nRows;
	}

	@Override
	public int getNCols() {
		return Constants.nCols;
	}

}
