package engine;

import java.util.ArrayList;
import java.util.List;

import board.Board;


public class DeciderEpsilonGreedy extends ExploitExplore {

	public DeciderEpsilonGreedy(int minDepth, int maxDepth) {
		super(minDepth, maxDepth);
	}

	@Override
	public int getNextMove(Board board, double[] weights, double trust) {
		int bootstrapMove = super.getNextMove(board, weights, trust);
		if (bootstrapMove != -1) return bootstrapMove;
		
		double r = Math.random();
		
		/*
		 * Given the constant epsilon, we want a function in trust such that f(trust=0) = 1, and f(trust=1) = epsilon
		 * The linear function satisfying this is f(t) = -(1-epsilon)*t + 1
		 */
		if (r < 1 - (1-Constants.epsilon)*trust) {
			List<Integer> legalMoves = new ArrayList<Integer>(Constants.nCols);
			for(int i=0; i<Constants.nCols; ++i)
				if (board.isLegalMove(i))
					legalMoves.add(i);
			return legalMoves.get((int)Math.floor(Math.random()*legalMoves.size()));
		}

		return miniMax.getMaxColumnScore(board, weights).getCol();
		
	}

}
