package engine;

import board.Board;



public class DeciderStochastic extends ExploitExplore {
	
	public DeciderStochastic(int minDepth, int maxDepth) {
		super(minDepth, maxDepth);
	}

	@Override
	public int getNextMove(Board board, double[] weights, double trust) {
		int bootstrapMove = super.getNextMove(board, weights, trust);
		if (bootstrapMove != -1) return bootstrapMove;
		
		Double[] columnScores = miniMax.getColumnScores(board, weights);

		double total = 0, runningTotal = 0;
		
		/*
		 * Shift Method
		 */
/*
		double min = Double.POSITIVE_INFINITY;
		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] != null && columnScores[i] < min) min = columnScores[i];
		}

		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			
			double positiveScore = (columnScores[i]-min+1)*trust;
			columnScores[i] = positiveScore;
			total += positiveScore;
		}
*/		
		/*
		 * Bounded Exponent Method
		 * 
		 */
		/*
		double min = Double.POSITIVE_INFINITY,
				max = Double.NEGATIVE_INFINITY;
		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			if (columnScores[i] < min) min = columnScores[i];
			if (columnScores[i] > max) max = columnScores[i];
		}
		
		max = Math.max(Math.abs(min), Math.abs(max));
		// now all the scores lie in the interval [-max,max]
		// we want a base such that base^max <= Constants.maximalExponent
		
		double base = Math.pow(Constants.maximalExponent, 1/max);
		
		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			
			double positiveScore = Math.pow(base, columnScores[i]*trust);
			columnScores[i] = positiveScore;
			total += positiveScore;
		}*/

		/*
		 * Shift & Bounded Exponent Method
		 */
		
		double min = Double.POSITIVE_INFINITY,
				max = Double.NEGATIVE_INFINITY;
		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			if (columnScores[i] < min) min = columnScores[i];
			if (columnScores[i] > max) max = columnScores[i];
			runningTotal += columnScores[i];
		}
		
		double avg = runningTotal/Constants.nCols,
				norm = Math.abs(max-avg); // avoid exploding the exponent in the positive direction; incredibly small exponents will only result in small values, which are fine.
		
		// if we're stretching when we normalise, then just leave the values as they are
		if (norm < Constants.maximalExponent) // this includes the case of norm==0 which would result in dividing by zero. in case they're all the same score, we don't want to divide by 0. This happens for the first iteration with initial weights of zero.
			norm = 1;
		else
			norm /= Constants.maximalExponent;

		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			
			// all exponents will now lie in [-stretch,stretch]
			double positiveScore = Math.exp((columnScores[i]-avg)/norm *trust);
			columnScores[i] = positiveScore;
			total += positiveScore;
		}
		
		runningTotal = 0;
		
		/*
		 * Regular Exponent Method
		 * 
		 */

		/*for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			
			double positiveScore = Math.pow(Math.E, columnScores[i]*trust);
			columnScores[i] = positiveScore;
			total += positiveScore;
		}*/

		double r = Math.random();
		
		for(int i=0; i<Constants.nCols; ++i) {
			if (columnScores[i] == null) continue;
			
			runningTotal += columnScores[i];
			if (r < runningTotal/total)
				return i;
		}

		// should never get here, but Java needs a definite return statement
		return -1;
		
	}

}
