package runtime;

import board.Board;
import engine.Constants;
import engine.DeciderEpsilonGreedy;
import engine.DeciderStochastic;
import engine.ExploitExplore;

public class PlayerSarsa implements IPlayer {


	private ExploitExplore nextMoveDecider;
	private double[] weights, weightsFirstPlayer, weightsSecondPlayer;
	private Board board;

	public PlayerSarsa(double[] weightsFirstPlayer, double[] weightsSecondPlayer, boolean isStochastic, String board) {
		this.weightsFirstPlayer = weightsFirstPlayer;
		this.weightsSecondPlayer = weightsSecondPlayer;
		nextMoveDecider = isStochastic ?
				new DeciderStochastic(Constants.minDepthLearning, Constants.maxDepthLearning) :
				new DeciderEpsilonGreedy(Constants.minDepthLearning, Constants.maxDepthLearning);
		this.board = Manager.getBoard(board);
 
	}
	
	@Override
	public int getMove() {
		int move = nextMoveDecider.getNextMove(board, weights, 1);
		board.setMove(move, false);
		return move;
	}

	@Override
	public void setOpponentsMove(int move) {
		board.setMove(move, true);
	}

	@Override
	public void start(boolean doIStart) {
		board.clear();
		weights = doIStart ? weightsFirstPlayer : weightsSecondPlayer;
	}

	@Override
	public void finish(int winner) {
	}
	
}
